// QCodeGenerator.java
//
// Informatics 102 Spring 2012
// Code Example
//
// The job of this class is to take an QScript object, describing the
// contents of an Q script, and generate a Java class that implements
// it.  Notice that the code we're generating matches the pattern we discovered
// in the "Part1" version of this code example.  See ExampleCommandRouter.java
// in "Part1" for a look at the pattern.  (One minor difference is that I'm
// fully qualifying names with their packages, to be sure that there aren't
// issues with importing or accessing classes in the runtime library.)

package inf102.q.compiler;


public class QCodeGenerator
{
	// We'll use this constant to be sure that we're using the appropriate
	// newline sequence at the end of each line; Windows and Mac/Unix/Linux
	// use different sequences.
	private static final String LINE_SEPARATOR =
		System.getProperty("line.separator");
	
	
	public String generateCode(QScript script)
	{
		StringBuilder code = new StringBuilder();
		
		println(code,
			"public class " + script.getRouterName() + "CommandRouter"
			+ " implements inf102.q.lib.CommandRouter");

		println(code, "{");
		println(code, "    public byte[] routeCommand(int commandByte, byte[] parameterBytes)");
		println(code, "    {");
		println(code, "        switch (commandByte)");
		println(code, "        {");
		
		for (QCommand command : script.getCommands())
		{
			println(code,
				"        case " + command.getCommandByte()
				+ ":    // " + command.getCommandHandlerName());
			
			println(code,
				"            return new " + command.getCommandHandlerName() + "CommandHandler"
				+ "().handleCommand(parameterBytes);");
		}
		
		println(code, "        default:");
		println(code, "            return new byte[0];");
		println(code, "        }");
		println(code, "    }");
		println(code, "}");
		
		return code.toString();
	}
	
	
	private void println(StringBuilder code, String line)
	{
		code.append(line + LINE_SEPARATOR);
	}
}
