// QParser.java
//
// Informatics 102 Spring 2012
// Code Example
//
// This class implements a Q parser.  It is much like previous parsers
// we've built in code examples, with one important difference: its job is
// not only to decide that a Q script is syntactically legal, but
// also to build us an object that conveniently represents all the information
// in the script.  Note that individual parseXXX() methods are returning
// objects that represent this information, when applicable.  Also, note
// that I split the expect() method from previous examples into two methods:
//
// * check(), which requires the current token to be of a particular type,
//   but does not advance the scanner to the next token
// * expect(), which does what it did before

package inf102.q.compiler;

import java.io.IOException;
import java.util.ArrayList;


public class QParser
{
	private QScanner scanner;
	
	
	public QParser(QScanner scanner)
	{
		this.scanner = scanner;
	}
	
	
	public QScript parse()
	{
		QScript script = parseScript();
		
		if (scanner.getCurrentTokenType() != QTokenType.END_OF_INPUT)
		{
			throw new QParseException("End of input expected");
		}
		
		return script;
	}
	
	
	// Script -> router Identifier '{' CommandList '}'
	private QScript parseScript()
	{
		expect(QTokenType.ROUTER);
		check(QTokenType.IDENTIFIER);

		String routerName = scanner.getCurrentToken();
		scanner.advance();
		
		expect(QTokenType.LEFT_BRACE);
		
		ArrayList<QCommand> commands = parseCommandList();
		
		expect(QTokenType.RIGHT_BRACE);
		
		return new QScript(routerName, commands);
	}
	
	
	// CommandList -> Command*
	private ArrayList<QCommand> parseCommandList()
	{
		ArrayList<QCommand> commands = new ArrayList<QCommand>();

		while (scanner.getCurrentTokenType() != QTokenType.RIGHT_BRACE)
		{
			commands.add(parseCommand());
		}
		
		return commands;
	}
	
	
	// Command -> Byte Identifier
	private QCommand parseCommand()
	{
		try
		{
			check(QTokenType.BYTE);
			int commandByte = Integer.parseInt(scanner.getCurrentToken());
			scanner.advance();

			check(QTokenType.IDENTIFIER);
			String commandHandlerName = scanner.getCurrentToken();
			scanner.advance();
			
			return new QCommand(commandByte, commandHandlerName);
		}
		catch (NumberFormatException e)
		{
			throw new QParseException(
				"Invalid command byte " + scanner.getCurrentToken());
		}		
	}
	
	
	private void expect(QTokenType tokenType)
	{
		check(tokenType);
		scanner.advance();
	}


	private void check(QTokenType tokenType)
	{
		if (scanner.getCurrentTokenType() != tokenType)
		{
			throw new QParseException(tokenType.toString() + " expected");
		}
	}
}
