// QScanner.java
//
// Informatics 102 Spring 2012
// Code Example
//
// This is a scanner for the Q language.

package inf102.q.compiler;

import java.io.IOException;
import java.io.Reader;


public class QScanner
{
	private Reader reader;
	private QTokenType currentTokenType;
	private String currentToken;
	private int nextChar;
	private static final int NO_MORE_CHARS = -1;
	
	
	public QScanner(Reader reader)
	{
		this.reader = reader;
		advanceToNextChar();
		advance();
	}
	
	
	public QTokenType getCurrentTokenType()
	{
		return currentTokenType;
	}
	
	
	public String getCurrentToken()
	{
		return currentToken;
	}
	
	
	public void advance()
	{
		consumeWhitespaceIfAny();
		
		if (noMoreChars())
		{
			currentTokenType = QTokenType.END_OF_INPUT;
			currentToken = "";
			return;
		}
		
		if (nextChar == '{')
		{
			currentTokenType = QTokenType.LEFT_BRACE;
			currentToken = "{";
			advanceToNextChar();
			return;
		}
		else if (nextChar == '}')
		{
			currentTokenType = QTokenType.RIGHT_BRACE;
			currentToken = "}";
			advanceToNextChar();
			return;
		}
		else if (Character.isDigit(nextChar))
		{
			currentToken = "";
			
			do
			{
				currentToken += (char) nextChar;
				advanceToNextChar();
			}
			while (Character.isDigit(nextChar));
			
			currentTokenType = QTokenType.BYTE;
			return;
		}
		else if (Character.isLetter(nextChar))
		{
			currentToken = "";
			
			do
			{
				currentToken += (char) nextChar;
				advanceToNextChar();
			}
			while (Character.isLetter(nextChar));
			
			if (currentToken.equals("router"))
			{
				currentTokenType = QTokenType.ROUTER;
				return;
			}
			else
			{
				currentTokenType = QTokenType.IDENTIFIER;
				return;
			}
		}
		else
		{
			throw new QScanException("Invalid character: " + (char) nextChar);
		}
	}
	
	
	private void consumeWhitespaceIfAny()
	{
		while (!noMoreChars() && Character.isWhitespace(nextChar))
		{
			advanceToNextChar();
		}
	}
	
	
	private void advanceToNextChar()
	{
		try
		{
			nextChar = reader.read();
		}
		catch (IOException e)
		{
			throw new QScanException("I/O error");
		}
	}


	private boolean noMoreChars()
	{
		return nextChar == NO_MORE_CHARS;
	}
}
